/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global $ */
/* @depend QfqEvents.js */

var QfqNS = QfqNS || {};

(function (n) {
    'use strict';

    n.Form = function (formId) {
        this.formId = formId;

        if (!document.forms[this.formId]) {
            throw new Error("Form '" + formId + "' does not exist.");
        }

        this.formChanged = false;
        this.$form = $(document.forms[this.formId]);
        this.$form.on("change", this.changeHandler.bind(this));

        // On text input elements, we specifically bind this events, in order to update the formChanged property
        // immediately, not only after loosing focus.
        this.$form.find("input[type=text]").on("input paste", this.changeHandler.bind(this));
    };

    /**
     *
     * @param event
     *
     * @private
     */
    n.Form.prototype.changeHandler = function (event) {
        this.formChanged = true;
        n.EventEmitter.emitEvent('form.changed', n.EventEmitter.makePayload(this, null));
        // REMOVE: this.userFormChangeHandlers.call(this);
    };

    n.Form.prototype.getFormChanged = function () {
        return this.formChanged;
    };

    n.Form.prototype.resetFormChanged = function () {
        this.formChanged = false;
        n.EventEmitter.emitEvent('form.reset', n.EventEmitter.makePayload(this, null));
        // REMOVE: this.userResetHandlers.call(this);
    };

    n.Form.prototype.submitTo = function (to) {
        $.post(to, this.$form.serialize())
            .done(this.ajaxSuccessHandler.bind(this))
            .fail(this.submitFailureHandler.bind(this));
    };

    n.Form.prototype.serialize = function () {
        return this.$form.serialize();
    };

    /**
     *
     * @param data
     * @param textStatus
     * @param jqXHR
     *
     * @private
     */
    n.Form.prototype.ajaxSuccessHandler = function (data, textStatus, jqXHR) {
        n.EventEmitter.emitEvent('form.submit.successful',
            n.EventEmitter.makePayload(this, data, {
                textStatus: textStatus,
                jqXHR: jqXHR
            }));
    };

    /**
     *
     *
     * @private
     */
    n.Form.prototype.submitFailureHandler = function (jqXHR, textStatus, errorThrown) {
        n.EventEmitter.emitEvent('form.submit.failed', n.EventEmitter.makePayload(this, null, {
            textStatus: textStatus,
            errorThrown: errorThrown,
            jqXHR: jqXHR
        }));
        // REMOVE: this.userSubmitFailureHandlers.call(this, textStatus, jqXHR, errorThrown);
    };

})(QfqNS);
